﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using System.Diagnostics;
using MandelbrotDemo;

namespace TestProject1
{
    // Currently no assertions, just tracing time...
    [TestClass]
    public class MandelbrotTests
    {
        int nx = 1000;
        int ny = 1000;

        [TestInitialize]
        public void Setup()
        {
            Parallel.For(0, 10, i => { Trace.WriteLine(i); });
        }

        [TestMethod]
        public void TestSolver()
        {
            var ms = new SolverFrame();
            ms.Solver = new MandelbrotSolver();
            ms.Width = nx;
            ms.Height = ny;

            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [TestMethod]
        public void ParallelSolver()
        {
            var ms = new SolverFrame();
            ms.Solver = new ParallelMandelbrotSolver();
            ms.Width = nx;
            ms.Height = ny;
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [TestMethod]
        public void Parallel2Solver()
        {
            var ms = new SolverFrame();
            ms.Solver = new Parallel2MandelbrotSolver();
            ms.Width = nx;
            ms.Height = ny;
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [TestMethod]
        public void Parallel3Solver()
        {
            var ms = new SolverFrame();
            ms.Solver = new Parallel3MandelbrotSolver();
            ms.Width = nx;
            ms.Height = ny;
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [TestMethod]
        public void ComparisonRange()
        {
            for (int i = 100; i <= 2000; i += 200)
            {
                var ms = new SolverFrame();
                ms.Width = ms.Height = i;
                ms.Solver = new MandelbrotSolver();
                var w = new Stopwatch();
                w.Start();
                ms.Update();
                long t = w.ElapsedMilliseconds;

                var msp = new SolverFrame();

                ms.Width = ms.Height = i;
                ms.Solver = new ParallelMandelbrotSolver();
                Stopwatch wp = new Stopwatch();
                wp.Start();
                msp.Update();
                long tp = wp.ElapsedMilliseconds;

                Trace.WriteLine(String.Format("{0}\t{1}\t{2}", i, t, tp));
                ms = null;
                msp = null;
            }
        }

    }
}
