﻿using System;
using System.Diagnostics;
using System.Threading;
using NUnit.Framework;

namespace MandelbrotDemo
{
    // Currently no assertions, just tracing time...
    [TestFixture]
    public class MandelbrotTests
    {
        int nx = 2000;
        int ny = 2000;

        [TestFixtureSetUp]
        public void Setup()
        {
            Parallel.For(0, 10, i => { Trace.WriteLine(i); });
        }

        [Test]
        public void TestSolver()
        {
            var ms = new SolverFrame();
            ms.Width = nx;
            ms.Height = ny;
            ms.Solver = new MandelbrotSolver();
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [Test]
        public void ParallelSolver()
        {
            var ms = new SolverFrame();
            ms.Width = nx;
            ms.Height = ny;
            ms.Solver = new ParallelMandelbrotSolver();
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [Test]
        public void Parallel2Solver()
        {
            var ms = new SolverFrame();
            ms.Width = nx;
            ms.Height = ny;
            ms.Solver = new Parallel2MandelbrotSolver();
            var w = new Stopwatch();
            w.Start();
            ms.Update();
            Trace.WriteLine(String.Format("{0} ms", w.ElapsedMilliseconds));
        }

        [Test]
        public void ComparisonRange()
        {
            Trace.WriteLine(String.Format("{0}\t{1}\t{2}\t{3}\t{4}", "MAXIT", "SIZE", "T1 [ms]", "T2 [ms]", "T1/T2"));
            int[] maxitSet = new int[] { 32, 64, 128, 256, 512, 1024 };
            int[] sizeSet = new int[] { 10, 20, 50, 100, 500, 750, 1000, 1500, 2000, 4000 };
            double[,] results = new double[maxitSet.Length, sizeSet.Length];

            var m1 = new SolverFrame();
            m1.Solver = new MandelbrotSolver();
            var m2 = new SolverFrame();
            m2.Solver = new ParallelMandelbrotSolver();

            for (int j = 0; j < maxitSet.Length; j++)
            {
                int maxit = m1.MaxIterations = m2.MaxIterations = maxitSet[j];

                for (int i = 0; i < sizeSet.Length; i++)
                {
                    int size = sizeSet[i];

                    // Non-parallel version
                    m1.Width = m1.Height = size;
                    var w1 = new Stopwatch();
                    w1.Start();
                    m1.Update();
                    double t1 = 1000.0 * w1.ElapsedTicks / Stopwatch.Frequency;

                    // Parallel version
                    m2.Width = m2.Height = size;
                    var w2 = new Stopwatch();
                    w2.Start();
                    m2.Update();
                    double t2 = 1000.0 * w2.ElapsedTicks / Stopwatch.Frequency;

                    results[j, i] = (double)t1 / t2;

                    Trace.WriteLine(String.Format("{0}\t{1}\t{2:0.0}\t{3:0.0}\t{4:0.00}", maxit, size, t1, t2, results[j, i]));
                }
            }
        }

    }

}
